import { jsx } from 'react/jsx-runtime';
import { createContext, useContext, useRef, useEffect, useLayoutEffect } from 'react';
import { EditorCache, createSQLEditorInstance } from '@tidbcloud/tisqleditor';

const EditorCacheContext = createContext(null);
const useEditorCacheContext = () => {
    const context = useContext(EditorCacheContext);
    const cacheRef = useRef(new EditorCache());
    useEffect(() => {
        return () => {
            cacheRef.current.clearEditors();
        };
    }, []);
    // if you don't need to edit multiple SQL files,
    // then you don't need to use EditorCacheProvider in the top
    if (!context) {
        // throw new Error('useEditorCacheContext must be used within a provider')
        console.warn(`useEditorCacheContext: no find EditorCacheProvider, so the cache is not shared, can't support multiple CodeMirror instances`);
        return cacheRef.current;
    }
    return context;
};
function EditorCacheProvider(props) {
    const cacheRef = useRef(new EditorCache());
    useEffect(() => {
        return () => {
            cacheRef.current.clearEditors();
        };
    }, []);
    return (jsx(EditorCacheContext.Provider, { value: cacheRef.current, children: props.children }));
}

function SQLEditor({ className, editorId, theme, sqlConfig, ...rest }) {
    const editorContainerRef = useRef(null);
    const cacheCtx = useEditorCacheContext();
    useLayoutEffect(() => {
        if (!editorContainerRef.current)
            return;
        let editorInst = cacheCtx.getEditor(editorId);
        if (!editorInst) {
            editorInst = createSQLEditorInstance({
                editorId,
                theme,
                sqlConfig,
                ...rest
            });
            cacheCtx.addEditor(editorId, editorInst);
        }
        editorContainerRef.current.appendChild(editorInst.editorView.dom);
        editorInst.editorView.focus();
        return () => {
            if (editorContainerRef.current && editorInst) {
                editorContainerRef.current.removeChild(editorInst.editorView.dom);
            }
        };
    }, [editorId]);
    // use `useLayoutEffect` to avoid flicker
    useLayoutEffect(() => {
        cacheCtx.getEditor(editorId)?.changeTheme(theme ?? []);
    }, [editorId, theme]);
    useEffect(() => {
        cacheCtx.getEditor(editorId)?.changeSQLConfig(sqlConfig ?? {});
    }, [editorId, sqlConfig]);
    return jsx("div", { className: className, ref: editorContainerRef });
}

export { EditorCacheProvider, SQLEditor, useEditorCacheContext };
